@@ -12,6 +12,26 @@
 const HighsInt dev_run = 0;
 const double double_equal_tolerance = 1e-5;
 
+static void userCallback(const int callback_type, const char* message,
+			 const HighsCallbackDataOut* data_out,
+			 HighsCallbackDataIn* data_in,
+			 void* user_callback_data) {
+  // Extract the double value pointed to from void* user_callback_data
+  const double local_callback_data = user_callback_data == NULL ? -1 : *(double*)user_callback_data;
+
+  if (callback_type == kHighsCallbackLogging) {
+    if (dev_run) printf("userCallback(%11.4g): %s\n", local_callback_data, message);
+  } else if (callback_type == kHighsCallbackMipImprovingSolution) {
+    if (dev_run) printf("userCallback(%11.4g): improving solution with objective = %g\n", local_callback_data, data_out->objective_function_value);
+  } else if (callback_type == kHighsCallbackMipLogging) {
+    if (dev_run) printf("userCallback(%11.4g): MIP logging\n", local_callback_data);
+    data_in->user_interrupt = 1;
+  } else if (callback_type == kHighsCallbackMipInterrupt) {
+    if (dev_run) printf("userCallback(%11.4g): MIP interrupt\n", local_callback_data);
+    data_in->user_interrupt = 1;
+  }
+}
+
 HighsInt intArraysEqual(const HighsInt dim, const HighsInt* array0, const HighsInt* array1) {
   for (HighsInt ix = 0; ix < dim; ix++) if (array0[ix] != array1[ix]) return 0;
   return 1;
@@ -55,52 +75,6 @@ void version_api() {
   }
 }
 
-void minimal_api() {
-  HighsInt num_col = 2;
-  HighsInt num_row = 2;
-  HighsInt num_nz = 4;
-  HighsInt a_format = kHighsMatrixFormatRowwise;
-  HighsInt sense = kHighsObjSenseMinimize;
-  double offset = 0;
-  HighsInt i;
-
-  double cc[2] = {1.0, -2.0};
-  double cl[2] = {0.0, 0.0};
-  double cu[2] = {10.0, 10.0};
-  double rl[2] = {0.0, 0.0};
-  double ru[2] = {2.0, 1.0};
-  HighsInt a_start[3] = {0, 2, 4};
-  HighsInt a_index[4] = {0, 1, 0, 1};
-  double a_value[4] = {1.0, 2.0, 1.0, 3.0};
-
-  double* cv = (double*)malloc(sizeof(double) * num_col);
-  double* cd = (double*)malloc(sizeof(double) * num_col);
-  double* rv = (double*)malloc(sizeof(double) * num_row);
-  double* rd = (double*)malloc(sizeof(double) * num_row);
-
-  HighsInt* cbs = (HighsInt*)malloc(sizeof(HighsInt) * num_col);
-  HighsInt* rbs = (HighsInt*)malloc(sizeof(HighsInt) * num_row);
-
-  HighsInt model_status;
-
-  HighsInt return_status = Highs_lpCall(num_col, num_row, num_nz, a_format, sense, offset,
-					cc, cl, cu, rl, ru, a_start, a_index, a_value, cv,
-					cd, rv, rd, cbs, rbs, &model_status);
-  assert( return_status == kHighsStatusOk );
-
-  if (dev_run) {
-    for (i = 0; i < num_col; i++) 
-      printf("x%"HIGHSINT_FORMAT" = %lf\n", i, cv[i]);
-  }
-
-  free(cv);
-  free(cd);
-  free(rv);
-  free(rd);
-  free(cbs);
-  free(rbs);
-}
-
 void minimal_api_lp() {
   // This illustrates the use of Highs_call, the simple C interface to
   // HiGHS. It's designed to solve the general LP problem
@@ -159,7 +133,7 @@ void minimal_api_lp() {
   const HighsInt num_col = 2;
   const HighsInt num_row = 3;
   const HighsInt num_nz = 5;
-  HighsInt a_format = 1;
+  HighsInt a_format = kHighsMatrixFormatColwise;
   HighsInt sense = kHighsObjSenseMinimize;
   double offset = 0;
 
@@ -186,11 +160,11 @@ void minimal_api_lp() {
   HighsInt model_status;
 
   HighsInt return_status = Highs_lpCall(num_col, num_row, num_nz, a_format,
-				    sense, offset, col_cost, col_lower, col_upper, row_lower, row_upper,
-				    a_start, a_index, a_value,
-				    col_value, col_dual, row_value, row_dual,
-				    col_basis_status, row_basis_status,
-				    &model_status);
+					sense, offset, col_cost, col_lower, col_upper, row_lower, row_upper,
+					a_start, a_index, a_value,
+					col_value, col_dual, row_value, row_dual,
+					col_basis_status, row_basis_status,
+					&model_status);
 
   assert( return_status == kHighsStatusOk );
 
@@ -234,7 +208,7 @@ void minimal_api_mip() {
   const HighsInt num_col = 3;
   const HighsInt num_row = 2;
   const HighsInt num_nz = 6;
-  HighsInt a_format = 1;
+  HighsInt a_format = kHighsMatrixFormatColwise;
   HighsInt sense = kHighsObjSenseMinimize;
   double offset = 0;
 
@@ -260,23 +234,24 @@ void minimal_api_mip() {
   HighsInt return_status;
 
   return_status = Highs_mipCall(num_col, num_row, num_nz, a_format,
-					 sense, offset, col_cost, col_lower, col_upper, row_lower, row_upper,
-					 a_start, a_index, a_value,
-					 integrality,
-					 col_value, row_value,
-					 &model_status);
-  // Should return error
+				sense, offset, col_cost, col_lower, col_upper, row_lower, row_upper,
+				a_start, a_index, a_value,
+				integrality,
+				col_value, row_value,
+				&model_status);
+  // Should return error, with model status not set
   assert( return_status == kHighsStatusError );
+  assert( model_status == kHighsModelStatusNotset );
 
   // Correct integrality
   integrality[num_col-1] = kHighsVarTypeInteger;
 
   return_status = Highs_mipCall(num_col, num_row, num_nz, a_format,
-					 sense, offset, col_cost, col_lower, col_upper, row_lower, row_upper,
-					 a_start, a_index, a_value,
-					 integrality,
-					 col_value, row_value,
-					 &model_status);
+				sense, offset, col_cost, col_lower, col_upper, row_lower, row_upper,
+				a_start, a_index, a_value,
+				integrality,
+				col_value, row_value,
+				&model_status);
   // Should return OK
   assert( return_status == kHighsStatusOk );
 
@@ -349,6 +324,35 @@ void minimal_api_qp() {
   free(col_value);
 }
 
+void minimal_api_illegal_lp() {
+  const double inf = 1e30;
+  HighsInt num_col = 2;
+  HighsInt num_row = 1;
+  HighsInt num_nz = 2;
+  HighsInt a_format = kHighsMatrixFormatRowwise;
+  HighsInt sense = kHighsObjSenseMinimize;
+  double offset = 0;
+  double col_cost[2] = {0.0, -1.0};
+  double col_lower[2] = {-inf, -inf};
+  double col_upper[2] = {inf, inf};
+  double row_lower[1] = {-inf};
+  double row_upper[1] = {2};
+  HighsInt a_start[1] = {0};
+  HighsInt a_index[2] = {0, -1}; // Illegal index
+  double a_value[2] = {1.0, 1.0};
+
+  HighsInt model_status;
+  HighsInt return_status = Highs_lpCall(num_col, num_row, num_nz, a_format,
+					sense, offset, col_cost, col_lower, col_upper, row_lower, row_upper,
+					a_start, a_index, a_value,
+					NULL, NULL, NULL, NULL, 
+					NULL, NULL, 
+					&model_status);
+  // Should return error, with model status not set
+  assert( return_status == kHighsStatusError );
+  assert( model_status == kHighsModelStatusNotset );
+}
+
 void full_api() {
   void* highs = Highs_create();
 
@@ -965,9 +969,9 @@ void full_api_qp() {
   HighsInt q_dim = 1;
   HighsInt q_num_nz = 1;
   HighsInt q_format = kHighsHessianFormatTriangular;
-  HighsInt* q_start = (HighsInt*)malloc(sizeof(HighsInt*) * q_dim);
-  HighsInt* q_index = (HighsInt*)malloc(sizeof(HighsInt*) * q_num_nz);
-  double* q_value = (double*)malloc(sizeof(double*) * q_num_nz);
+  HighsInt* q_start = (HighsInt*)malloc(sizeof(HighsInt) * q_dim);
+  HighsInt* q_index = (HighsInt*)malloc(sizeof(HighsInt) * q_num_nz);
+  double* q_value = (double*)malloc(sizeof(double) * q_num_nz);
   q_start[0] = 0;
   q_index[0] = 0;
   q_value[0] = 2.0;
@@ -1113,6 +1117,124 @@ void full_api_qp() {
 
 }
 
+void pass_presolve_get_lp() {
+  // Form and solve the LP
+  // Min    f  = 2x_0 + 3x_1
+  // s.t.                x_1 <= 6
+  //       10 <=  x_0 + 2x_1 <= 14
+  //        8 <= 2x_0 +  x_1
+  // 0 <= x_0 <= 3; 1 <= x_1
+
+  void* highs;
+
+  highs = Highs_create();
+  const double kHighsInf = Highs_getInfinity(highs);
+  HighsInt model_status;
+  HighsInt return_status;
+  
+  Highs_setBoolOptionValue(highs, "output_flag", dev_run);
+  HighsInt a_format = kHighsMatrixFormatColwise;
+  HighsInt sense = kHighsObjSenseMinimize;
+  double offset = 0;
+  // Define the column costs, lower bounds and upper bounds
+
+  const HighsInt num_col = 2;
+  const HighsInt num_row = 3;
+  const HighsInt num_nz = 5;
+
+  double col_cost[2] = {2.0, 3.0};
+  double col_lower[2] = {0.0, 1.0};
+  double col_upper[2] = {3.0, kHighsInf};
+  // Define the row lower bounds and upper bounds
+  double row_lower[3] = {-kHighsInf, 10.0, 8.0};
+  double row_upper[3] = {6.0, 14.0, kHighsInf};
+  HighsInt a_start[2] = {0, 2};
+  HighsInt a_index[5] = {1, 2, 0, 1, 2};
+  double a_value[5] = {1.0, 2.0, 1.0, 2.0, 1.0};
+
+  return_status = Highs_passLp(highs, num_col, num_row, num_nz, a_format, sense, offset,
+			       col_cost, col_lower, col_upper,
+			       row_lower, row_upper,
+			       a_start, a_index, a_value);
+  assert( return_status == kHighsStatusOk );
+
+  return_status = Highs_presolve(highs);
+  assert( return_status == kHighsStatusOk );
+  for (HighsInt k = 0; k < 2; k++) {
+    // Loop twice: once for col-wise; once for row-wise
+    HighsInt presolved_num_col = Highs_getPresolvedNumCol(highs);
+    HighsInt presolved_num_row = Highs_getPresolvedNumRow(highs);
+    HighsInt presolved_num_nz = Highs_getPresolvedNumNz(highs);
+    HighsInt presolved_a_format = k == 0 ? kHighsMatrixFormatColwise : kHighsMatrixFormatRowwise;
+    HighsInt presolved_sense;
+    double presolved_offset;
+    double* presolved_col_cost = (double*)malloc(sizeof(double) * presolved_num_col);
+    double* presolved_col_lower = (double*)malloc(sizeof(double) * presolved_num_col);
+    double* presolved_col_upper = (double*)malloc(sizeof(double) * presolved_num_col);
+    double* presolved_row_lower = (double*)malloc(sizeof(double) * presolved_num_row);
+    double* presolved_row_upper = (double*)malloc(sizeof(double) * presolved_num_row);
+    HighsInt* presolved_a_start = (HighsInt*)malloc(sizeof(HighsInt) * (presolved_num_col+1));
+    HighsInt* presolved_a_index = (HighsInt*)malloc(sizeof(HighsInt) * presolved_num_nz);
+    double* presolved_a_value = (double*)malloc(sizeof(double) * presolved_num_nz);
+  
+    return_status = Highs_getPresolvedLp(highs, presolved_a_format,
+					 &presolved_num_col, &presolved_num_row, &presolved_num_nz,
+					 &presolved_sense, &presolved_offset,
+					 presolved_col_cost, presolved_col_lower, presolved_col_upper,
+					 presolved_row_lower, presolved_row_upper,
+					 presolved_a_start, presolved_a_index, presolved_a_value, NULL);
+    assert( return_status == kHighsStatusOk );
+    // Solve the presolved LP within a local version of HiGHS
+    void* local_highs;
+    local_highs = Highs_create();
+    Highs_setBoolOptionValue(local_highs, "output_flag", dev_run);
+    Highs_setStringOptionValue(local_highs, "presolve", "off");
+    return_status = Highs_passLp(local_highs,
+				 presolved_num_col, presolved_num_row, presolved_num_nz,
+				 presolved_a_format, presolved_sense, presolved_offset,
+				 presolved_col_cost, presolved_col_lower, presolved_col_upper,
+				 presolved_row_lower, presolved_row_upper,
+				 presolved_a_start, presolved_a_index, presolved_a_value);
+    assert( return_status == kHighsStatusOk );
+    return_status = Highs_run(local_highs);
+    
+    double* col_value = (double*)malloc(sizeof(double) * num_col);
+    double* col_dual = (double*)malloc(sizeof(double) * num_col);
+    double* row_dual = (double*)malloc(sizeof(double) * num_row);
+
+    return_status = Highs_getSolution(local_highs, col_value, col_dual, NULL, row_dual);
+    assert( return_status == kHighsStatusOk );
+
+    return_status = Highs_postsolve(highs, col_value, col_dual, row_dual);
+    assert( return_status == kHighsStatusOk );
+
+    model_status = Highs_getModelStatus(highs);
+    assert( model_status == kHighsModelStatusOptimal );
+    
+    // With just the primal solution, optimality cannot be determined
+
+    return_status = Highs_postsolve(highs, col_value, NULL, NULL);
+    assert( return_status == kHighsStatusWarning );
+
+    model_status = Highs_getModelStatus(highs);
+    assert( model_status == kHighsModelStatusUnknown );
+
+    free(presolved_col_cost);
+    free(presolved_col_lower);
+    free(presolved_col_upper);
+    free(presolved_row_lower);
+    free(presolved_row_upper);
+    free(presolved_a_start);
+    free(presolved_a_index);
+    free(presolved_a_value);
+    free(col_value);
+    free(col_dual);
+    free(row_dual);
+
+    
+  }
+}
+
 void options() {
   void* highs = Highs_create();
   if (!dev_run) Highs_setBoolOptionValue(highs, "output_flag", 0);
@@ -1321,6 +1443,60 @@ void test_ranging() {
   free(row_bound_dn_in_var);
   free(row_bound_dn_ou_var);
 
+  Highs_destroy(highs);
+}
+
+void test_callback() {
+  HighsInt num_col = 7;
+  HighsInt num_row = 1;
+  HighsInt num_nz = num_col;
+  HighsInt a_format = kHighsMatrixFormatRowwise;
+  HighsInt sense = kHighsObjSenseMaximize;
+  double offset = 0;
+  double col_cost[7] = {8, 1, 7, 2, 1, 2, 1};
+  double col_lower[7] = {0, 0, 0, 0, 0, 0, 0};
+  double col_upper[7] = {1, 1, 1, 1, 1, 1, 1};
+  double row_lower[1] = {0};
+  double row_upper[1] = {28};
+  HighsInt a_start[2] = {0, 7};
+  HighsInt a_index[7] = {0, 1, 2, 3, 4, 5, 6};
+  double a_value[7] = {9, 6, 7, 9, 7, 9, 9};
+  HighsInt integrality[7] = {kHighsVarTypeInteger, kHighsVarTypeInteger,
+			     kHighsVarTypeInteger, kHighsVarTypeInteger,
+			     kHighsVarTypeInteger, kHighsVarTypeInteger,
+			     kHighsVarTypeInteger};
+
+  void* highs;
+  highs = Highs_create();
+  Highs_setBoolOptionValue(highs, "output_flag", dev_run);
+  Highs_passMip(highs, num_col, num_row, num_nz, a_format, sense, offset,
+		col_cost, col_lower, col_upper,
+		row_lower, row_upper,
+		a_start, a_index, a_value,
+		integrality);
+  
+  Highs_setCallback(highs, userCallback, NULL);
+  Highs_startCallback(highs, kHighsCallbackLogging);
+  Highs_startCallback(highs, kHighsCallbackMipInterrupt);
+  Highs_run(highs);
+  double objective_function_value;
+  Highs_getDoubleInfoValue(highs, "objective_function_value", &objective_function_value);
+  double inf = Highs_getInfinity(highs);
+  assertDoubleValuesEqual("objective_function_value", objective_function_value, inf);
+  Highs_stopCallback(highs, kHighsCallbackMipInterrupt);
+  Highs_run(highs);
+  Highs_getDoubleInfoValue(highs, "objective_function_value", &objective_function_value);
+  assertDoubleValuesEqual("objective_function_value", objective_function_value, 17);
+
+  double user_callback_data = inf;
+  void* p_user_callback_data = (void*)(&user_callback_data);
+  
+  Highs_setCallback(highs, userCallback, p_user_callback_data);
+  Highs_clearSolver(highs);
+  Highs_startCallback(highs, kHighsCallbackMipImprovingSolution);
+  Highs_run(highs);
+  
+  Highs_destroy(highs);
 }
 
 /*
@@ -1369,8 +1545,9 @@ void test_setSolution() {
 }
 */
 int main() {
+  minimal_api_illegal_lp();
+  test_callback();
   version_api();
-  minimal_api();
   full_api();
   minimal_api_lp();
   minimal_api_mip();
@@ -1379,6 +1556,7 @@ int main() {
   full_api_lp();
   full_api_mip();
   full_api_qp();
+  pass_presolve_get_lp();
   options();
   test_getColsByRange();
   test_passHessian();
