@@ -1,13 +1,14 @@
 #include "Avgas.h"
 #include "HCheckConfig.h"
 #include "Highs.h"
+#include "SpecialLps.h"
 #include "catch.hpp"
 #include "lp_data/HighsLpUtils.h"
 #include "util/HighsRandom.h"
 #include "util/HighsUtils.h"
 
 const bool dev_run = false;
-const double inf = kHighsInf;
+// const double inf = kHighsInf;
 const double double_equal_tolerance = 1e-5;
 void HighsStatusReport(const HighsLogOptions& log_options, std::string message,
                        HighsStatus status);
@@ -1842,3 +1843,88 @@ void messageReportMatrix(const char* message, const HighsInt num_col,
               message);
   reportMatrix(log_options, message, num_col, num_nz, start, index, value);
 }
+
+TEST_CASE("mod-duplicate-indices", "[highs_data]") {
+  Highs highs;
+  highs.setOptionValue("output_flag", dev_run);
+  const std::string filename =
+      std::string(HIGHS_DIR) + "/check/instances/avgas.mps";
+  highs.readModel(filename);
+  std::vector<double> lower = {0, 0, 0, 0};
+  std::vector<HighsInt> set0 = {5, 2, 7, 3};
+  std::vector<double> lower0 = {1, 1, 1, 1};
+  std::vector<double> upper0 = {1, 1, 1, 1};
+  std::vector<HighsInt> set1 = {5, 2, 7, 3, 2};
+  std::vector<double> lower1 = {0, 0, 0, 0, 0};
+  std::vector<double> upper1 = {1, 1, 1, 1, 1};
+  REQUIRE(highs.changeColsBounds(HighsInt(set0.size()), set0.data(),
+                                 lower0.data(),
+                                 upper0.data()) == HighsStatus::kOk);
+  highs.run();
+  double objective1 = highs.getInfo().objective_function_value;
+  // Reverting the change with duplicate index should fail
+  REQUIRE(highs.changeColsBounds(HighsInt(set1.size()), set1.data(),
+                                 lower1.data(),
+                                 upper1.data()) == HighsStatus::kError);
+  // Reverting the change without duplicate index should be OK
+  REQUIRE(highs.changeColsBounds(HighsInt(set0.size()), set0.data(),
+                                 lower.data(),
+                                 upper0.data()) == HighsStatus::kOk);
+  highs.run();
+  double objective0 = highs.getInfo().objective_function_value;
+  REQUIRE(objective0 < objective1);
+  REQUIRE(objective0 == -7.75);
+}
+
+TEST_CASE("resize-integrality", "[highs_data]") {
+  Highs highs;
+  highs.setOptionValue("output_flag", dev_run);
+  SpecialLps special_lps;
+  HighsLp lp;
+  HighsModelStatus require_model_status;
+  double optimal_objective;
+  special_lps.distillationLp(lp, require_model_status, optimal_objective);
+  HighsInt original_num_col = lp.num_col_;
+  for (HighsInt k = 0; k < 4; k++) {
+    // k = 0: Add continuous column to LP, so final integrality.size() should be
+    // 0
+    //
+    // k = 1: Add continuous column to IP, so final integrality.size() should be
+    // full
+    //
+    // k = 2: Add integer column to LP, so final integrality.size() should be
+    // full
+    //
+    // k = 3: Add integer column to IP, so final integrality.size() should be
+    // full
+    if (k == 1 || k == 3) {
+      lp.integrality_.assign(original_num_col, HighsVarType::kInteger);
+    } else {
+      lp.integrality_.clear();
+    }
+    REQUIRE(highs.passModel(lp) == HighsStatus::kOk);
+    REQUIRE(highs.getNumCol() == original_num_col);
+    double cost = 0.0;
+    double lower = 0.0;
+    double upper = 1.0;
+    highs.addCols(1, &cost, &lower, &upper, 0, nullptr, nullptr, nullptr);
+    const std::vector<HighsVarType>& integrality = highs.getLp().integrality_;
+    if (k == 0 || k == 2) {
+      // Model is LP
+      REQUIRE(int(integrality.size()) == 0);
+    } else {
+      // Model is MIP
+      REQUIRE(int(integrality.size()) == int(original_num_col + 1));
+    }
+    if (k >= 2)
+      REQUIRE(highs.changeColIntegrality(2, HighsVarType::kInteger) ==
+              HighsStatus::kOk);
+    if (k == 0) {
+      // Model is LP
+      REQUIRE(int(integrality.size()) == 0);
+    } else {
+      // Model is MIP
+      REQUIRE(int(integrality.size()) == int(original_num_col + 1));
+    }
+  }
+}
