@@ -150,7 +150,7 @@ public class HighsLpSolver : IDisposable
 
     private bool _disposed;
 
-    private const string highslibname = "libhighs";
+    private const string highslibname = "highs";
 
     [DllImport(highslibname)]
     private static extern int Highs_call(
@@ -909,8 +909,9 @@ public class HighsLpSolver : IDisposable
             DualBound = this.GetValueOrFallback(HighsLpSolver.Highs_getDoubleInfoValue, "mip_dual_bound", double.NaN),
             ObjectiveValue = this.GetValueOrFallback(HighsLpSolver.Highs_getDoubleInfoValue, "objective_function_value", double.NaN),
             NodeCount = this.GetValueOrFallback(HighsLpSolver.Highs_getInt64InfoValue, "mip_node_count", 0l),
-            IpmIterationCount = this.GetValueOrFallback(HighsLpSolver.Highs_getIntInfoValue, "simplex_iteration_count", 0),
-            SimplexIterationCount = this.GetValueOrFallback(HighsLpSolver.Highs_getIntInfoValue, "ipm_iteration_count", 0),
+            IpmIterationCount = this.GetValueOrFallback(HighsLpSolver.Highs_getIntInfoValue, "ipm_iteration_count", 0),
+            SimplexIterationCount = this.GetValueOrFallback(HighsLpSolver.Highs_getIntInfoValue, "simplex_iteration_count", 0),
+            PdlpIterationCount = this.GetValueOrFallback(HighsLpSolver.Highs_getIntInfoValue, "pdlp_iteration_count", 0),
         };
         return info;
     }
@@ -1001,6 +1002,11 @@ public class SolutionInfo
     /// </summary>
     public int IpmIterationCount { get; set; }
 
+    /// <summary>
+    /// Gets or sets the PDLP iteration count.
+    /// </summary>
+    public int PdlpIterationCount { get; set; }
+
     /// <summary>
     /// Gets or sets the MIP gap.
     /// </summary>
